import * as tslib_1 from "tslib";
/** @jsx jsx */
import { jsx } from "@emotion/core";
import * as React from "react";
import { animated, useSpring } from "react-spring";
import PropTypes from "prop-types";
import { useMeasure } from "./Hooks/use-measure";
import { useUid } from "./Hooks/use-uid";
import { Text } from "./Text";
export function useCollapse(defaultShow, press) {
    if (defaultShow === void 0) { defaultShow = false; }
    if (press === void 0) { press = false; }
    var _a = React.useState(defaultShow), show = _a[0], setShow = _a[1];
    var id = useUid();
    function onClick() {
        setShow(!show);
    }
    React.useEffect(function () {
        setShow(defaultShow);
    }, [defaultShow]);
    return {
        show: show,
        setShow: setShow,
        id: id,
        buttonProps: {
            onClick: press ? undefined : onClick,
            onPress: press ? onClick : undefined,
            "aria-controls": id,
            "aria-expanded": show ? true : false
        },
        collapseProps: {
            id: id,
            show: show
        }
    };
}
/**
 * Hide and reveal content with an animation. Supports dynamic
 * heights.
 */
export var AnimateCollapse = function (_a) {
    var _b;
    var children = _a.children, id = _a.id, show = _a.show, divStyle = _a.divStyle, other = tslib_1.__rest(_a, ["children", "id", "show", "divStyle"]);
    var ref = React.useRef(null);
    var bounds = useMeasure(ref).bounds;
    var prevShow = usePrevious(show);
    //高度 动画副作用，导致计算问题？ 旧的to: { height: show ? bounds.height : 0 },
    //这个height实际是一个对象的。  const { animationObj } = useSpring ；注意两个height不是一个东西。
    //这里useSpring({）的参数 实际是初始化用的，后面输出的对象height.value实际是动画编制的起始数值，而不是目标数值。
    //[问题！] 这里的useSpring（）没有处理内部DIV高度变化的副作用；打印机场景div高度会缩放调整，应该被缩小的<animated.div却没有被缩小。
    var height = useSpring({
        from: { height: 0 },
        to: { height: show ? bounds.height : 0 },
        immediate: prevShow !== null && prevShow === show
    }).height;
    //实际打印预览会捕获3次的render这里，Collapse-捕获height=，前面2次纸张缩放调整，缩小了，div就会变矮化，后面第三次却是屏幕的。
    //只好先预留大一点的高度。 有待改进。动画可能切割打印。
    var dynamicHeight = (bounds.height > (height && height.value)) ? (bounds.height) : (height && height.value) > 0 ? (height && height.value) : undefined;
    return (jsx("div", { css: [
            (_b = {
                    height: show ? dynamicHeight : 0
                },
                _b["@media not print"] = {
                    display: show ? undefined : 'none',
                },
                _b["@media print"] = {
                    overflow: "hidden",
                },
                _b),
        ] },
        jsx(animated.div, tslib_1.__assign({ id: id, style: { height: height }, css: {
                willChange: "height, opacity",
            } }, other),
            jsx("div", { ref: ref, css: divStyle }, children))));
};
/**
 * Hide and reveal content with an animation. Supports dynamic heights.
 * 针对两种方式取舍display:'none', 和 show&& (<div>{children}</div>)比较，还是逻辑隐藏更好，若遇到耗时子组件时表现更好。
 */
export var Collapse = function (_a) {
    var children = _a.children, id = _a.id, show = _a.show, divStyle = _a.divStyle, _b = _a.noAnimated, noAnimated = _b === void 0 ? false : _b, other = tslib_1.__rest(_a, ["children", "id", "show", "divStyle", "noAnimated"]);
    return (jsx(React.Fragment, null, noAnimated ? (jsx("div", tslib_1.__assign({ id: id, "aria-hidden": !show, css: {
            display: show ? undefined : 'none',
            overflow: "hidden",
        } }, other),
        jsx(LazyToggleViewer, { show: show, css: divStyle }, children)))
        :
            (jsx(AnimateCollapse, { id: id, show: show, divStyle: divStyle }, children))));
};
Collapse.propTypes = {
    children: PropTypes.node,
    id: PropTypes.string.isRequired,
    show: PropTypes.bool.isRequired
};
export function usePrevious(value) {
    var ref = React.useRef(null);
    React.useEffect(function () {
        ref.current = value;
    }, [value]);
    return ref.current;
}
//切换隐藏显示得内容： 初始化show=false情形=启动懒加载。是取长补短的模式。
//若放在逻辑&&方式{show&&()}的组件下或路由后再进入呈现的子组件，就失去效果，父组件被要求重新渲染！就等于树的根节点决定枝条节点命运。
//这样来看该组件放在层级太低的组件位置，可能没有意义，要搞就得放在较高组件层次的位置上。路由RouterLink本身就是逻辑&&方式的。
export var LazyToggleViewer = function (_a) {
    var show = _a.show, children = _a.children, other = tslib_1.__rest(_a, ["show", "children"]);
    var _b = React.useState(false), mounted = _b[0], setMounted = _b[1];
    if (!mounted) {
        if (show)
            setMounted(true);
        //这里反正底下组件还未加载，可以这么做
        return null;
    }
    return (jsx("div", tslib_1.__assign({ css: { display: show ? undefined : 'none' } }, other), children));
};
LazyToggleViewer.propTypes = {
    children: PropTypes.node,
    show: PropTypes.bool.isRequired
};
//性能测试专用的。
export var TestRenderLoad = function (_a) {
    var _b = _a.loop, loop = _b === void 0 ? 3000 : _b, _c = _a.num, num = _c === void 0 ? 4 : _c, _d = _a.txt, txt = _d === void 0 ? '针对渲染次数做优化，性能测试专用的！' : _d, other = tslib_1.__rest(_a, ["loop", "num", "txt"]);
    console.log(txt);
    return (jsx("div", tslib_1.__assign({ css: { display: 'none' } }, other), Array.from(new Array(loop)).map(function (a, i) {
        return jsx(Text, { key: i }, Array.from(new Array(num)).map(function (a, i) { return jsx(Text, { key: i }, txt); }));
    })));
};
