import * as tslib_1 from "tslib";
/** @jsx jsx */
import { jsx } from "@emotion/core";
import * as React from "react";
import { useGestureResponder } from "react-gesture-responder";
import { useSpring } from "react-spring";
import PropTypes from "prop-types";
import { safeBind } from "./Hooks/compose-bind";
/**
 * A scroll view with some helpers, including:
 *  - smooth scrolling
 *  - gesture claiming
 * @param param0
 * @param componentRef  改成 ref
 */
//export const ScrollView= React.forwardRef(ScrollViewForward);   // 报警！嵌套RefForwardingComponent 的forwardRef;
export var ScrollView = React.forwardRef(function (_a, ref) {
    var overflowY = _a.overflowY, children = _a.children, overflowX = _a.overflowX, innerRef = _a.innerRef, _b = _a.scrollAnimationConfig, scrollAnimationConfig = _b === void 0 ? { tension: 190, friction: 15, mass: 0.2 } : _b, other = tslib_1.__rest(_a, ["overflowY", "children", "overflowX", "innerRef", "scrollAnimationConfig"]);
    var scrRef = React.useRef(null);
    /**
     * A spring for animating scroll positions
     */
    var _c = useSpring(function () {
        return {
            config: scrollAnimationConfig,
            from: { x: 0, y: 0 },
            to: { x: 0, y: 0 },
            onFrame: function (animated) {
                if (overflowX && scrRef.current) {
                    scrRef.current.scrollLeft = animated.x;
                }
                if (overflowY && scrRef.current) {
                    scrRef.current.scrollTop = animated.y;
                }
            }
        };
    }), setScroll = _c[1];
    /**
     * Expose an imperate scrollTo method
     */
    React.useImperativeHandle(ref, function () { return ({
        scrollTo: function (x, y) {
            if (scrRef.current) {
                var from = {
                    x: scrRef.current.scrollLeft,
                    y: scrRef.current.scrollTop
                };
                setScroll({
                    from: from,
                    to: { x: x, y: y },
                    reset: true
                });
            }
        }
    }); }, [setScroll]);
    /**
     * Use a pan responder to determine if our scrollview should
     * claim the responder (and cancel other gestures out). Only
     * enabled on touch devices.
     */
    var bind = useGestureResponder({
        onStartShouldSet: function () { return false; },
        onTerminationRequest: function () { return false; },
        onMoveShouldSet: function (_a) {
            var initial = _a.initial, initialDirection = _a.initialDirection, xy = _a.xy;
            if (initialDirection[0] !== 0 && overflowX) {
                return true;
            }
            if (initialDirection[1] !== 0 && overflowY) {
                return true;
            }
            return false;
        }
    }, {
        enableMouse: false
    }).bind;
    // 不可把innerRef?  与 ref?  都绑定到内部第二个div，scrRef上。
    return (jsx("div", tslib_1.__assign({ className: "ScrollView" }, other, bind),
        jsx("div", tslib_1.__assign({ className: "ScrollView__scroll-containerr", css: {
                transform: "translateZ(0)",
                overflowX: overflowX ? "scroll" : undefined,
                overflowY: overflowY ? "scroll" : undefined,
                WebkitOverflowScrolling: "touch"
            } }, safeBind({ ref: innerRef }, { ref: scrRef }, other)), children)));
});
ScrollView.propTypes = {
    overflowY: PropTypes.bool,
    children: PropTypes.node,
    overflowX: PropTypes.bool,
    scrollAnimationConfig: PropTypes.shape({
        tension: PropTypes.number,
        mass: PropTypes.number,
        friction: PropTypes.number
    })
};
